package hdl

import (
	"applet/app/db"
	"applet/app/e"
	"applet/app/lib/validate"
	"applet/app/md"
	"applet/app/svc"
	"applet/app/svc/sys_cfg"
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/aliyun"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/enum"
	md3 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/tidwall/gjson"
	"time"
)

// SmsSend
// @Summary      发送短信
// @Tags         登录注册
// @Description  发送短信
// @Accept       json
// @Produce      json
// @Param        req 	body      md.SmsSendReq          true   "注册参数"
// @Success      200   {string}   			"成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/smsSend [post]
func SmsSend(c *gin.Context) {
	var req md.SmsSendReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}

	userDb := implement.NewUserDb(db.Db)
	if utils.InArr(req.Type, []string{"h5Register", "wechatBindPhone", "login"}) == false {
		user, err := userDb.UserGetOneByParams(map[string]interface{}{
			"key":   "phone",
			"value": req.Mobile,
		})
		if err != nil {
			e.OutErr(c, 400, err)
			return
		}
		if user == nil {
			e.OutErr(c, 400, e.NewErr(400, "账号不存在"))
			return
		}
	}
	if req.Type == "h5Register" {
		user, _ := userDb.UserGetOneByParams(map[string]interface{}{
			"key":   "phone",
			"value": req.Mobile,
		})
		if user != nil {
			e.OutErr(c, 400, e.NewErr(400, "您已注册过，请前往下载APP"))
			return
		}
	}
	//param := map[string]string{
	//	"lot_number":     req.LotNumber,
	//	"gen_time":       req.GenTime,
	//	"captcha_output": req.CaptchaOutput,
	//	"pass_token":     req.PassToken,
	//}
	////校验图形验证码
	//id, key := svc.AliyunCaptchBase(c)
	//err = aliyun.AliyunCheckCaptcha(id, key, param)
	//if err != nil {
	//	e.OutErr(c, 400, e.NewErr(400, "图形验证码校验失败"))
	//	return
	//}
	err = svc.CommSmsSend(c, req.Mobile, req.Type)
	if err != nil {
		if err.Error() == "请1分钟后再试" {
			e.OutErr(c, 400, e.NewErr(400, "请1分钟后再试"))
			return
		}
		e.OutErr(c, 400, e.NewErr(400, "发送失败"))
		return
	}
	//发送短信
	e.OutSuc(c, "success", nil)
	return
}

// FastLogin
// @Summary      一键登录
// @Tags         登录注册
// @Description  一键登录
// @Accept       json
// @Produce      json
// @Param        req 	body      md.FastLoginReq          true   "注册参数"
// @Success      200   {object}   md.LoginResponse			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/fastLogin [post]
func FastLogin(c *gin.Context) {
	var req md.FastLoginReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	utils.FilePutContents("FastLogin", utils.SerializeStr(req))
	data := svc.AliyunSmsBase(c, "")
	//获取手机
	phone, err := aliyun.AliyunFastPhone(data["aliyun_sms_id"], data["aliyun_sms_secret"], req.Token)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "获取手机号失败"))
		return
	}
	var reqs = md.RegisterReq{
		Mobile: phone,
		Type:   "app",
	}
	commReq(c, reqs)
}

// WechatLogin
// @Summary      微信登陆
// @Tags         登录注册
// @Description  微信登陆
// @Accept       json
// @Produce      json
// @Param        req 	body      md.WechatLoginReq          true   "注册参数"
// @Success      200   {object}   md.LoginResponse			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/wechatLogin [post]
func WechatLogin(c *gin.Context) {
	var req md.WechatLoginReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	utils.FilePutContents("WechatLogin", utils.SerializeStr(req))
	if req.WechatCode == "" {
		e.OutErr(c, 400, e.NewErr(400, "获取微信信息失败"))
		return
	}
	//获取openid
	wechatUrl := "https://api.weixin.qq.com/sns/oauth2/access_token?appid=%s&secret=%s&code=%s&grant_type=authorization_code"
	wechatUrl = fmt.Sprintf(wechatUrl, svc.GetSysCfgStr("wechat_appid"), svc.GetSysCfgStr("wechat_secret"), req.WechatCode)
	wechatResp, err := utils.CurlGet(wechatUrl, nil)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "获取微信信息失败"))
		return
	}
	utils.FilePutContents("WechatLogin", utils.SerializeStr(wechatResp))
	req.OpenID = gjson.Get(string(wechatResp), "openid").String()
	req.UnionId = gjson.Get(string(wechatResp), "unionid").String()
	wechatToken := gjson.Get(string(wechatResp), "access_token").String()
	wechatInfoUrl := "https://api.weixin.qq.com/sns/userinfo?access_token=%s&openid=%s"
	wechatInfoUrl = fmt.Sprintf(wechatInfoUrl, wechatToken, req.OpenID)
	wechatInfoResp, _ := utils.CurlGet(wechatInfoUrl, nil)
	req.Avatar = gjson.Get(string(wechatInfoResp), "headimgurl").String()
	req.Nickname = gjson.Get(string(wechatInfoResp), "nickname").String()
	sex := gjson.Get(string(wechatInfoResp), "sex").Int()
	req.Sex = utils.Int64ToStr(sex)
	if req.OpenID == "" {
		e.OutErr(c, 400, e.NewErr(400, "登录失败"))
		return
	}
	param := map[string]interface{}{
		"key":   "open_id",
		"value": req.OpenID,
	}
	if req.UnionId != "" {
		param = map[string]interface{}{
			"key":   "union_id",
			"value": req.UnionId,
		}
	}
	// 如果内存缓存中没有，则从 数据库查询
	newUserDb := implement.NewUserDb(db.Db)
	userModel, err := newUserDb.UserGetOneByParams(param)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "登录失败"))
		return
	}
	token := ""
	if userModel != nil {
		if userModel.OpenId == "" {
			userModel.OpenId = req.OpenID
		}
		if userModel.Avatar == "" {
			userModel.Avatar = req.Avatar
		}
		if userModel.Nickname == "" {
			userModel.Nickname = req.Nickname
		}
		now := time.Now()
		userModel.LastLoginAt = now.Format("2006-01-02 15:04:05")
		userModel.LastLoginIp = utils.GetIP(c.Request)
		userModel.UpdateAt = now.Format("2006-01-02 15:04:05")
		newUserDb.UpdateUser(userModel, "open_id,avatar,nickname,last_login_ip,last_login_at,update_at")
		svc.UserImeiAdd(c, userModel.Id)
		token, err = svc.HandleLoginToken(userModel)
		if err != nil {
			e.OutErr(c, e.ERR, err.Error())
			return
		}
	} else {
		cache.SetEx("wechat:"+req.WechatCode, utils.SerializeStr(req), 3600)
	}
	e.OutSuc(c, md.LoginResponse{Token: token}, nil)
	return
}

// FindPassword
// @Summary      找回密码
// @Tags         登录注册
// @Description  找回密码
// @Accept       json
// @Produce      json
// @Param        req 	body      md.FindPasswordReq          true   "注册参数"
// @Success      200   string   			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/findPassword [post]
func FindPassword(c *gin.Context) {
	var req md.FindPasswordReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	userDb := implement.NewUserDb(db.Db)
	user, err := userDb.UserGetOneByParams(map[string]interface{}{
		"key":   "phone",
		"value": req.Mobile,
	})
	if user == nil {
		e.OutErr(c, 400, e.NewErr(400, "手机号不存在"))
		return
	}
	//校验短信
	err = svc.CommSmsCheck(c, req.Mobile, req.Code)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "验证码错误,请重试"))
		return
	}
	user.Password = utils.Md5(req.Password)
	userDb.UpdateUser(user, "password")
	e.OutSuc(c, "success", nil)
	return
}

// Register
// @Summary      注册
// @Tags         登录注册
// @Description  注册
// @Accept       json
// @Produce      json
// @Param        req 	body      md.RegisterReq          true   "注册参数"
// @Success      200   {object}   md.LoginResponse			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/register [post]
func Register(c *gin.Context) {
	var req md.RegisterReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	//校验短信
	err = svc.CommSmsCheck(c, req.Mobile, req.Code)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "验证码错误,请重试"))
		return
	}
	commReq(c, req)
}

// WechatRegister
// @Summary      注册
// @Tags         登录注册
// @Description  注册
// @Accept       json
// @Produce      json
// @Param        req 	body      md.RegisterReq          true   "注册参数"
// @Success      200   {object}   md.LoginResponse			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/wechatRegister [post]
func WechatRegister(c *gin.Context) {
	var req md.RegisterReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	//校验短信
	err = svc.CommSmsCheck(c, req.Mobile, req.Code)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "验证码错误,请重试"))
		return
	}
	if req.WechatCode == "" {
		e.OutErr(c, 400, "获取微信信息失败")
		return
	}
	commReq(c, req)
}

// Login 登陆
// @Summary      登陆
// @Tags         登录注册
// @Description  登入
// @Accept       json
// @Produce      json
// @Param        req 	body      md.LoginReq          true   "用户名密码"
// @Success      200   {object}   md.LoginResponse				"token"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/login [post]
func Login(c *gin.Context) {
	var req md.LoginReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = validate.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	userDb := implement.NewUserDb(db.Db)
	user, err := userDb.UserGetOneByParams(map[string]interface{}{
		"key":   "phone",
		"value": req.Phone,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err)
		return
	}
	if user == nil {
		e.OutErr(c, e.ERR_NO_DATA, "账号不存在！")
		return
	}
	fmt.Println(utils.Md5(req.PassWord))
	if utils.Md5(req.PassWord) != user.Password {
		e.OutErr(c, e.ERR_INVALID_ARGS, "密码错误")
		return
	}
	//ip := utils.GetIP(c.Request)
	token, err := svc.HandleLoginToken(user)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	svc.UserImeiAdd(c, user.Id)
	e.OutSuc(c, md.LoginResponse{
		Token: token,
	}, nil)
	return
}

// 公共处理注册
func commReq(c *gin.Context, req md.RegisterReq) {
	now := time.Now()
	userDb := implement.NewUserDb(db.Db)
	user, err := userDb.UserGetOneByParams(map[string]interface{}{
		"key":   "phone",
		"value": req.Mobile,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err)
		return
	}

	ip := utils.GetIP(c.Request)
	//如果是微信绑定 这里要获取缓存
	if req.WechatCode != "" {
		wechatInfo, _ := cache.GetString("wechat:" + req.WechatCode)
		req.OpenID = gjson.Get(wechatInfo, "open_id").String()
		req.UnionId = gjson.Get(wechatInfo, "union_id").String()
		req.Avatar = gjson.Get(wechatInfo, "avatar").String()
		req.Nickname = gjson.Get(wechatInfo, "nickname").String()
		req.Sex = utils.StrToInt(gjson.Get(wechatInfo, "sex").String())
		if req.OpenID == "" {
			e.OutErr(c, 400, e.NewErr(400, "登录失败"))
			return
		}
		param := map[string]interface{}{
			"key":   "open_id",
			"value": req.OpenID,
		}
		if req.UnionId != "" {
			param = map[string]interface{}{
				"key":   "union_id",
				"value": req.UnionId,
			}
		}
		newUserDb := implement.NewUserDb(db.Db)
		userModel, _ := newUserDb.UserGetOneByParams(param)
		if userModel != nil && req.Mobile != userModel.Phone {
			e.OutErr(c, 400, "该手机号已绑定其他微信账号")
			return
		}
	}
	if user == nil {
		registerType := func() int {
			if req.Type == "H5" {
				return 2
			}
			return 1
		}()
		var parentUser *model.User
		var inviteCode string
		sysCfgDb := sys_cfg.NewSysCfgDb(db.Db)
		sysCfg := sysCfgDb.SysCfgFindWithDb(enum.AppInviteType, enum.AppInviteLength)
		inviteCode = svc.ReturnCode(utils.StrToInt(sysCfg[enum.AppInviteLength]), utils.StrToInt(sysCfg[enum.AppInviteType]), 0)
		if req.InviteCode != "" {
			// Todo::拥有上下级关系
			parentUser, err = userDb.GetUserByInviteCode(req.InviteCode)
			if err != nil {
				e.OutErr(c, e.ERR_SUP_NOTFOUND, "邀请码失效，请联系上级重新获取分享!")
				return
			}
			if parentUser == nil {
				parentUser, err = userDb.GetUserByCustomInviteCode(req.InviteCode)
				if err != nil || parentUser == nil {
					e.OutErr(c, e.ERR_SUP_NOTFOUND, "邀请码失效，请联系上级重新获取分享!")
					return
				}
			}
		}
		var parentUid = func() int64 {
			if parentUser != nil {
				return parentUser.Id
			}
			return 0
		}()
		if req.Nickname == "" && req.Mobile != "" {
			appName := svc.GetSysCfgStr("app_name")
			req.Nickname = appName + req.Mobile[len(req.Mobile)-4:]
		}
		if req.Avatar == "" {
			req.Avatar = svc.GetSysCfgStr("default_avatar")
		}
		NewUserLevelDb := implement.NewUserLevelDb(db.Db)
		userLevel, _ := NewUserLevelDb.UserLevelInIDescByWeightLowWithOne()
		level := 0
		if userLevel != nil {
			level = userLevel.Id
		}
		user = &model.User{
			Phone:            req.Mobile,
			UnionId:          req.UnionId,
			OpenId:           req.OpenID,
			Nickname:         req.Nickname,
			Avatar:           req.Avatar,
			Password:         "",
			Passcode:         "",
			Level:            level,
			InviteTotal:      0,
			State:            int(enum.UserStateForNormal),
			LastLoginIp:      ip,
			Sex:              req.Sex,
			ParentUid:        parentUid,
			SystemInviteCode: inviteCode,
			CustomInviteCode: "",
			Memo:             "",
			IsRealName:       0,
			RegisterType:     registerType,
			CreateAt:         now.Format("2006-01-02 15:04:05"),
			UpdateAt:         now.Format("2006-01-02 15:04:05"),
		}
		if req.Password != "" {
			user.Password = utils.Md5(req.Password)
		}

		insertAffected, err1 := userDb.UserInsert(user)
		if err1 != nil {
			e.OutErr(c, e.ERR_DB_ORM, err1)
			return
		}
		if insertAffected <= 0 {
			e.OutErr(c, e.ERR_DB_ORM, "新增用户数据失败")
			return
		}
		if parentUid > 0 {
			initLV := 1
			ur := new(model.UserRelate)
			ur.ParentUid = parentUid
			ur.Uid = user.Id
			ur.Level = initLV
			ur.InviteTime = time.Now().Format("2006-01-02 15:04:05")

			userRelateDb := implement.NewUserRelateDb(db.Db)
			_, err = userRelateDb.UserRelateInsert(ur)
			if err != nil {
				e.OutErr(c, e.ERR_DB_ORM, err)
				return
			}
			// 插入多级关联
			go svc.RoutineMultiRelate(ur.ParentUid, ur.Uid, initLV)
			//TODO::推入mq异步处理
			ch, err := rabbit.Cfg.Pool.GetChannel()
			if err != nil {
				e.OutErr(c, e.ERR_INIT_RABBITMQ, err1.Error())
				return
			}
			defer ch.Release()
			ch.Publish(md3.EggAppExchange, md3.AddPublicPlatoonUserRelationCommissionReq{RecommendUid: utils.Int64ToStr(parentUid), Uid: utils.Int64ToStr(user.Id)}, md3.EggRoutKeyForAddPublicPlatoonUserRelationCommission)
		}
		//TODO::推入mq处理
		ch, err := rabbit.Cfg.Pool.GetChannel()
		if err != nil {
			e.OutErr(c, e.ERR_INIT_RABBITMQ, err1.Error())
			return
		}
		defer ch.Release()
		ch.Publish(md3.EggAppExchange, md3.EggNewUserRegisterData{Uid: user.Id}, md3.EggNewUserRegister)
	}

	token, err := svc.HandleLoginToken(user)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}

	if req.Nickname != "" {
		user.Nickname = req.Nickname
	}
	if req.Avatar != "" {
		user.Avatar = req.Avatar
	}
	if req.Password != "" {
		user.Password = utils.Md5(req.Password)
	}

	// 微信
	if req.OpenID != "" && user.OpenId == "" {
		user.OpenId = req.OpenID
	}
	if req.UnionId != "" && user.UnionId == "" {
		user.UnionId = req.UnionId
	}
	// 登录信息
	user.LastLoginAt = now.Format("2006-01-02 15:04:05")
	user.LastLoginIp = ip
	user.UpdateAt = now.Format("2006-01-02 15:04:05")

	updateAffected, err1 := userDb.UpdateUser(user)
	if err1 != nil {
		e.OutErr(c, e.ERR_DB_ORM, err1)
		return
	}
	if updateAffected <= 0 {
		e.OutErr(c, e.ERR_DB_ORM, "更新用户数据失败")
		return
	}
	svc.UserImeiAdd(c, user.Id)
	e.OutSuc(c, md.LoginResponse{
		Token: token,
	}, nil)
	return
}
